// Helper for YahooUICalendar, copyright 2006, Fred Janon
// Version 1.0.0 Oct 25, 2006
/*
 Description:
 The YahooCalendar Helper is designed to make it easier to use the YahooUI YAHOO.widget.Calendar.
 It wraps the YAHOO.widget.Calendar in a YahooUICalendar object. The YahooUICalendar will create the
 the HTML structure to host a small calendar icon (that the user clicks to make the calendar visible)
 and the calendar itself. It will also link the calendar to the date field and optionally
 to a hidden field for server submission. The default date in the date field will be automatically
 formatted in the locale of the client browser. No need to worry about that on the server side.
 For example, if the date field contains "2006/11/24", the field will be changed to
 Friday, 24 November 2006 if the locale is set to Australia

 To use it, you need to include the following in your HTML page:
    <script type="text/javascript" src="yui_0.11.4/build/yahoo/yahoo-min.js"></script>
    <script type="text/javascript" src="yui_0.11.4/build/event/event-min.js"></script>
    <script type="text/javascript" src="yui_0.11.4/build/dom/dom-min.js"></script>
    <script type="text/javascript" src="yui_0.11.4/build/calendar/calendar-min.js"></script>
    <link type="text/css" rel="stylesheet" href="yui_0.11.4/build/calendar/assets/calendar.css" />
 and create the YahooCalendar object like this for example AFTER your HTML and before the </body> tag:
  <script type="text/javascript">
    var endDateCalendar = new YahooUICalendar("endDate", "endDateCalendar",null,"endDateSubmit","images/triangle_grey.png");
  </script>
 The default font size in the Yahoo Calendar is pretty big in 0.11.4
 You can change it by adding this style in your header after the Yahoo calendar.css link
    .yui-calendar {
    	font:70% sans-serif;
    }
You need to have a small icon where the user clicks to get the calendar to appear.
The default icon pathname is "images/defaultcalendar.gif"

You need also to download the YahooUI library and set the "src" path for the scripts above according
to the version you got and where you unzipped it. I tested it with yui_0.11.4.

Last, put this file in the path you set in your HTML file:
script type="text/javascript" src="js/YahooCalendarHelper.js"

The Calendar is locale aware, so you should get the proper date formatting according to the client
setting. The only fixed format is for the _fieldForSubmitId, it is always in "yyyy/mm/dd" format
so it can be parsed and store easily on the server.

Tested with IE6 and Firefox 1.5 on Windoz. Issues/suggestions: let me know at fjanon@yahoo.com
*/

/* API:

 function YahooUICalendar(_dateFieldId, _calendarContainerId, _minDate, _fieldForSubmitId, _iconPath)

  _dateFieldId: String, required. The SPAN or INPUT READONLY element id used as the date field.
  Using a SPAN makes it a readonly field with any style.
  Using an INPUT READONLY field makes it easy to generate on a JSP.
  Note: if the INPUT is not READONLY and the user changes the value in the INPUT,
  the calendar selected date is not updated accordingly, nor would be the field with _fieldForSubmitId.
  If you want an INPUT TEXT element (modifiable) then do not use a _fieldForSubmitId

  _calendarContainerId: String, required. The empty SPAN or DIV id used to host the calendar and its icon

  _minDate: String, optional. If a valid date is supplied, the calendar will disable any date prior to it.
  If null, all dates are allowed. If "" (empty string), no date allowed before the client's today date.

  _fieldForSubmitId: String, optional. Used to store the selected date in a year/month/day format, whatever
  locale is used. The field would be a hidden field with an id "_fieldForSubmitId" attribute,
  a name attribute(to be submitted to the server).

  _iconPath: String, URI, optional. Default: "images/" Path to the images: defaultcalendar.gif,
   callt.gif (left arrow), calrt.gif (right arrow).
   The last 2 images should be in the Yahoo "assets", you can jut copy them
   in the default "images/" folder at the same level as your HTML page

  Examples:

  var startDateCalendar = new YahooUICalendar("dateField", "calendarContainer","27 Oct 2006");
  => Create a YahooUICalendar in the HTML element with id="calendarContainer", the calendar gets/sets
  the date in the HTML element with the id="dateField". The first date allowed to be selected in
  the calendar is the "27 Oct 2006". For this to be useful, the HTML element would have to be an
  INPUT READONLY with a 'name' attribute like this:
  <input readonly id="dateField" name="endDate" type='text' value='25 December, 2006' size='30'>

  var endDateCalendar = new YahooUICalendar("endDate", "endDateCalendar",null,"endDateSubmit","images/");
*/

// Library functions used by the YahooCalendar object

function $ref(elementId)
    {
     var elt = document.getElementById(elementId);
//     if (elt == null) alert("$ref: '" + elementId + "' null!!"); // Debug
     return elt;
    }

function getDateYear(_date)
  {
   var year;
   if (_date.getFullYear)
    {
     year = _date.getFullYear();
    }
   else
    {
     year = 1900 + selectedDate.getYear();
    }
   return year;
  }

function getValidDate(dateString)
  {
  var timeInMillis = Date.parse(dateString);
  var aDate = new Date(timeInMillis);
  // Can't parse the date => use today's date
  if (isNaN(aDate)) aDate = new Date();
  return aDate;
  }

function getMonthYear(dateString)
    {
     var aDate = getValidDate(dateString);
     var day = aDate.getDate();
     var month = aDate.getMonth() + 1;
     var year = getDateYear(aDate);
     var monthYearStr = "" + month + "/" + year;
     return monthYearStr;
    }

function getMonthDayYear(dateString)
    {
     var aDate = getValidDate(dateString);
     var day = aDate.getDate();
     var month = aDate.getMonth() + 1;
     var year = getDateYear(aDate);
     var monthDayYearStr = "" + month + "/" + day + "/" + year;
     return monthDayYearStr;
    }

function getYearMonthDay(dateString)
    {
     var aDate = getValidDate(dateString);
     var day = aDate.getDate();
     var month = aDate.getMonth() + 1;
     var year = getDateYear(aDate);
     var yearMonthDayStr = "" + year + "/" + month + "/" + day;
     return yearMonthDayStr;
    }

function getElementText(elementId)
  {
   var elt;
   if (typeof(elementId) == 'string') elt = $ref(elementId);
    else elt = elementId;
    // Works for IE
   if (elt.innerText) return elt.innerText;
    // if element is an INPUT
   if (elt.value) return elt.value;
   // Works for IE and FF
   if (elt.hasChildNodes())
    {
     for (var i in elt.childNodes)
      {
       var child = elt.childNodes[i]
       if (child.nodeType == 3) // text node
        {
         return child.nodeValue;
        }
      }
    }
   if (elt.nodeType == 3)
    {
     if (elt.nodeValue)
      {
       return elt.nodeValue;
      }
    }
   return null; // Could not find the element value
  } // End of getElementText

function setElementText(elementId,newText)
  {
   var elt;
   if (typeof(elementId) == 'string') elt = $ref(elementId);
    else elt = elementId;
    // Works for IE
   if (elt.innerText)
    {
     elt.innerText = newText;
     return;
    }
   if (elt.value)
    {
     elt.value = newText;
    }
   // Works for IE and FF
   if (elt.hasChildNodes())
    {
     for (var i in elt.childNodes)
      {
       var child = elt.childNodes[i]
       if (child.nodeType == 3) // text node
        {
         child.nodeValue = newText;
         return;
        }
      }
    }
   if (elt.nodeType == 3)
    {
     if (elt.nodeValue)
      {
       elt.nodeValue = newText;
      }
    }
  } // End of getElementText

// The YahooCalendar object.
function YahooUICalendar(_dateFieldId, _calendarContainerId, _minDate, _fieldForSubmitId, _iconPath)
{
  /*
  Structure:
    -  a span or input with id = _dateFieldId
      + the initial value of the calendar selected date is taken from there
      + when the user selects a date in the Yahoo calendar, this span or input value is set to the calendar selected date
    - a span with id = _calendarContainerId to host the Yahoo calendar widget elements
      + the icon element is created dynamically by the function "createIconElement" before the calendar container
    The HTML structure should look like (replace the ids by yours):
    
    <span id="startDate">26 Nov 2006</span> <span id="startDateCalendar"/>
    then in a script after the HTML in the "body"
    var startDateCal = new YahooUICalendar("startDate","startDateCalendar");
    The generated structure will look like: see file "Calendar HTML Structure.html"
  */

  var dateFieldId = _dateFieldId;  // Required, element where the selected date will be set (input or span)
  var calendarContainerId = _calendarContainerId; // Required, element where the icon and calendar table will be displayed
  var calendarId = calendarContainerId + "_YahooCalendar"; // new div element created by new Yahoo.widget.Calendar
  var minDate = _minDate; // Optional
  var fieldForSubmitId = _fieldForSubmitId; // optional, field filled in with the selected date with the format "year,month,day"

  // Default folder where we expect to find the images for the calendar
  // defaultcalendar.gif, callt.gif (left arrow), calrt.gif (right arrow)
  var iconDefaultPath = "images/";
  var iconPath = _iconPath || iconDefaultPath; // _iconPath is optional
  if (iconPath.charAt(iconPath.length - 1) != '/') iconPath += '/';

  var initialSelectedDate;
  // Set the calendar container attributes
  var calendarContainer = $ref(calendarContainerId);
  calendarContainer.style.position = 'absolute';
  calendarContainer.style.display = 'none';
  // Create the IMG element
  var iconId = calendarContainerId + '_Icon';
  var iconElement = createIconElement(calendarContainer,iconId);
  iconElement.onclick = toggleCalendar; // link the IMG to the calendar
  // Creating the span for the Calendar dynamically doesn't work in IE. It messes up the position of the calendar and its elements

  var dateField = $ref(dateFieldId);
//  var dateString = dateField.innerHTML;
  var dateString = getElementText(dateFieldId);
  var monthYear = getMonthYear(dateString);
  var monthDayYear = getMonthDayYear(dateString);

// Format the date field to the locale
  var aDate = getValidDate(dateString);
  var newDateStr = aDate.toLocaleDateString();
  setElementText(dateFieldId,newDateStr);

  var calendarWidget = new YAHOO.widget.Calendar(calendarId,calendarContainerId,monthYear,monthDayYear);

  if (minDate)
    {
     if (minDate != "") calendarWidget.minDate = new Date(minDate); // Cannot select a date before minDate
    }
  else
    {
     if (minDate != null) calendarWidget.minDate = new Date(); // Cannot select a date before today
    }
  calendarWidget.onSelect = onDateSelect;
/////////////
// Kludge to fix the fixed path to akamai.net.
// Does not fix the problem for the 2Up calendar "us/my/bn/x_d.gif"
    calendarWidget.Config.Options.NAV_ARROW_LEFT = iconPath + "callt.gif"; // FJ
    calendarWidget.Config.Options.NAV_ARROW_RIGHT = iconPath + "calrt.gif"; // FJ
/////////////
  calendarWidget.render();

  var elementToHideId = calendarContainerId;
  var elementToHide = $ref(elementToHideId);

  // Optional field updated with the selected date with a [year,month,day] format for the server
  // The field should have a 'name' attribute to be submitted to the server
  var fieldForSubmit = null;
  if (fieldForSubmitId)
    {
     fieldForSubmit = $ref(fieldForSubmitId);
     // Copy the date field into the field to submit
     fieldForSubmit.value = getYearMonthDay(dateString);
    }
  return;

  // These methods/functions should be at the beginning of the YahooCalendar object
  // But it is more readable to have them here...
  function showCalendar()
    {
     elementToHide.style.display = 'block';
    }

  function hideCalendar()
    {
     elementToHide.style.display = 'none';
    }

  function calendarDisplayed()
    {
     if ((elementToHide.style.display == '') || (elementToHide.style.display == 'none')) return false;
     return true;
    }

	function toggleCalendar(event)
    {
     // If the calendar is visible, hide it and vice-versa
     if (calendarDisplayed())
      {
       hideCalendar();
       return;
      }
		 var pos = YAHOO.util.Dom.getXY(iconElement);
		 showCalendar();
		 YAHOO.util.Dom.setXY(elementToHide, [pos[0], pos[1] + iconElement.offsetHeight + 1]);
   }

  function createIconElement(calendarContainerElement,iconId)
    {
    // Add the img element before the calendar container for the Yahoo Calendar
    var img = document.createElement('img');
    img.id = iconId;
    img.src = iconPath + "defaultcalendar.gif";
    img.alt = "Calendar";
    img.style.verticalAlign = 'middle'; // setAttribute("style","") doesn't work properly
    var parent = calendarContainerElement.parentNode;
    parent.insertBefore(img,calendarContainerElement);
    return img;
    }

  function onDateSelect(selectedDatesArray)
      {
       // the param selectedDateArray doesn't work with the older YahooUI libs
       // so we use an API that works in all versions
       var selectedDate = calendarWidget.getSelectedDates()[0];
       // cal1.hide() doesn't work with the YahooUI lib prior to 0.11
       hideCalendar();
       setElementText(dateField,selectedDate.toLocaleDateString());
       if (fieldForSubmit != null)
        {
         var year = getDateYear(selectedDate);
         fieldForSubmit.value = year + '/' + (selectedDate.getMonth() + 1) + '/' + selectedDate.getDate();
        }
      }
} // End of YahooUICalendar
